/*
 * © 2009 David ‘Bombe’ Roden
 */
package net.pterodactylus.arachne.core;

import java.net.MalformedURLException;
import java.net.URL;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.concurrent.Executor;
import java.util.concurrent.Executors;
import java.util.logging.Level;
import java.util.logging.Logger;

import net.pterodactylus.arachne.parser.ParserFactory;
import de.ina.util.service.AbstractService;
import de.ina.util.thread.DumpingThreadFactory;
import de.ina.util.validation.Validation;

/**
 * Arachne’s core.
 *
 * @author David ‘Bombe’ Roden <bombe@pterodactylus.net>
 */
public class Core extends AbstractService {

	/** The logger. */
	private static final Logger logger = Logger.getLogger(Core.class.getName());

	//
	// PROPERTIES
	//

	/** The host of the freenet node. */
	private String nodeHost = "localhost";

	/** The port of the freenet node. */
	private int nodePort = 8888;

	//
	// INTERNAL MEMBERS
	//

	/** The parser factory. */
	private ParserFactory parserFactory = new ParserFactory();

	/** Thread pool for the URL fetches. */
	private Executor urlFetcherExecutor = Executors.newSingleThreadExecutor(new DumpingThreadFactory("URLFetcher-"));

	/** The current list of URLs to crawl. */
	private final List<Page> pages = new ArrayList<Page>();

	/** Already crawled pages. */
	private final Set<Page> crawledPages = new HashSet<Page>();

	//
	// ACCESSORS
	//

	/**
	 * Sets the host name of the node.
	 *
	 * @param nodeHost
	 *            The node’s host name
	 */
	public void setNodeHost(String nodeHost) {
		this.nodeHost = nodeHost;
	}

	//
	// ACTIONS
	//

	/**
	 * Adds the given URL to the list of pages to crawl.
	 *
	 * @param url
	 *            The URL to add
	 */
	public void addPage(URL url) {
		Validation.begin().isNotNull("url", url).check().isEqual("url.getHost()", url.getHost(), (Object) nodeHost).isEqual("url.getPort()", url.getPort(), nodePort).check();
		addPage(Page.fromURL(url));
	}

	/**
	 * Adds the given URL to the list of pages to crawl.
	 *
	 * @param url
	 *            The URL of the page to crawl
	 * @throws MalformedURLException
	 *             if the URL is not a valid URL
	 */
	public void addPage(String url) throws MalformedURLException {
		Validation.begin().isNotNull("url", (Object) url).check();
		addPage(new URL(url));
	}

	/**
	 * Adds the given page to the list of pages to crawl.
	 *
	 * @param page
	 *            The page to add
	 */
	public void addPage(Page page) {
		Validation.begin().isNotNull("page", page).check();
		if (!crawledPages.contains(page) && !pages.contains(page)) {
			pages.add(page);
			notifySyncObject();
		}
	}

	//
	// SERVICE METHODS
	//

	/**
	 * {@inheritDoc}
	 *
	 * @see de.ina.util.service.AbstractService#serviceRun()
	 */
	@Override
	protected void serviceRun() {
		while (!shouldStop()) {
			final Page nextPage;
			Page page = null;
			synchronized (syncObject) {
				while (!shouldStop() && pages.isEmpty()) {
					try {
						syncObject.wait();
					} catch (InterruptedException ie1) {
						/* ignore. */
					}
				}
				if (!shouldStop()) {
					page = pages.remove(0);
				}
			}
			if (shouldStop()) {
				break;
			}
			nextPage = page;
			URL nextURL = nextPage.toURL(nodeHost, nodePort);
			if (nextURL == null) {
				logger.log(Level.INFO, "Skipping “" + nextPage + "”.");
				continue;
			}
			final URLFetcher urlFetcher;
			try {
				logger.log(Level.INFO, "Fetching “" + nextURL + "”...");
				urlFetcher = new URLFetcher(parserFactory, nextURL);
				urlFetcherExecutor.execute(new Runnable() {

					@SuppressWarnings("synthetic-access")
					public void run() {
						urlFetcher.run();
						crawledPages.add(nextPage);
						for (Page page : urlFetcher.getCollectedPages()) {
							addPage(page);
						}
					}
				});
			} catch (NoSuchAlgorithmException nsae1) {
				logger.log(Level.SEVERE, "Could not get “SHA-256” message digest!", nsae1);
			}
		}
	}

}
